@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,10 +53,6 @@ def urljoin (parent, url, scheme):
     """
     if url.startswith(scheme+":"):
         return url
-    # work around a Python 2.6/3.1 bug cutting off characters when the URL
-    # begins with semicolon
-    if url.startswith(';'):
-        url = "./%s" % url
     return urlparse.urljoin(parent, url)
 
 
@@ -71,6 +67,17 @@ def url_norm (url, encoding=None):
         raise LinkCheckerError(msg)
 
 
+def getXmlText (parent, tag):
+    """Return XML content of given tag in parent element."""
+    elem = parent.getElementsByTagName(tag)[0]
+    # Yes, the DOM standard is awful.
+    rc = []
+    for node in elem.childNodes:
+        if node.nodeType == node.TEXT_NODE:
+            rc.append(node.data)
+    return ''.join(rc)
+
+
 class UrlBase (object):
     """An URL with additional information like validity etc."""
 
@@ -88,6 +95,7 @@ class UrlBase (object):
         "text/plain+linkchecker": "text",
         "text/plain+opera": "opera",
         "text/plain+chromium": "chromium",
+        "application/x-plist+safari": "safari",
     }
 
     # Set maximum file size for downloaded files in bytes.
@@ -131,6 +139,7 @@ class UrlBase (object):
         self.column = column
         self.name = name
         self.encoding = url_encoding
+        self.charset = None
         if self.base_ref:
             assert not urlutil.url_needs_quoting(self.base_ref), \
                    "unquoted base reference URL %r" % self.base_ref
@@ -227,24 +236,27 @@ class UrlBase (object):
 
     def set_title_from_content (self):
         """Set title of page the URL refers to.from page content."""
-        if self.valid:
-            try:
-                handler = linkparse.TitleFinder()
-            except tuple(ExcList):
-                return
-            parser = htmlsax.parser(handler)
-            handler.parser = parser
-            # parse
-            try:
-                parser.feed(self.get_content())
-                parser.flush()
-            except linkparse.StopParse, msg:
-                log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
-            # break cyclic dependencies
-            handler.parser = None
-            parser.handler = None
-            if handler.title:
-                self.title = handler.title
+        if not self.valid:
+            return
+        try:
+            handler = linkparse.TitleFinder()
+        except tuple(ExcList):
+            return
+        parser = htmlsax.parser(handler)
+        handler.parser = parser
+        if self.charset:
+            parser.encoding = self.charset
+        # parse
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
+        # break cyclic dependencies
+        handler.parser = None
+        parser.handler = None
+        if handler.title:
+            self.title = handler.title
 
     def is_parseable (self):
         """
@@ -293,6 +305,7 @@ class UrlBase (object):
         """
         Fill attributes from cache data.
         """
+        self.url = cache_data["url"]
         self.result = cache_data["result"]
         self.has_result = True
         anchor_changed = (self.anchor != cache_data["anchor"])
@@ -318,7 +331,8 @@ class UrlBase (object):
 
     def get_cache_data (self):
         """Return all data values that should be put in the cache."""
-        return {"result": self.result,
+        return {"url": self.url,
+                "result": self.result,
                 "warnings": self.warnings,
                 "info": self.info,
                 "valid": self.valid,
@@ -591,6 +605,8 @@ class UrlBase (object):
         handler = linkparse.MetaRobotsFinder()
         parser = htmlsax.parser(handler)
         handler.parser = parser
+        if self.charset:
+            parser.encoding = self.charset
         # parse
         try:
             parser.feed(self.get_content())
@@ -713,25 +729,34 @@ class UrlBase (object):
                 self.check_html()
             if self.aggregate.config["checkcss"] and self.is_css():
                 self.check_css()
-            if self.aggregate.config["checkhtmlw3"] and self.is_html():
-                self.check_html_w3()
-            if self.aggregate.config["checkcssw3"] and self.is_css():
-                self.check_css_w3()
             # check with clamav
             if self.aggregate.config["scanvirus"]:
                 self.scan_virus()
 
     def check_warningregex (self):
         """Check if content matches a given regular expression."""
-        warningregex = self.aggregate.config["warningregex"]
+        config = self.aggregate.config
+        warningregex = config["warningregex"]
         if not (warningregex and self.valid and self.is_parseable()):
             return
-        log.debug(LOG_CHECK, "checking content")
+        log.debug(LOG_CHECK, "checking content for warning regex")
         try:
-            match = warningregex.search(self.get_content())
-            if match:
-                self.add_warning(_("Found %(match)r in link contents.") %
-                   {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
+            content = self.get_content()
+            curpos = 0
+            curline = 1
+            # add warnings for found matches, up to the maximum allowed number
+            for num, match in enumerate(warningregex.finditer(content)):
+                # calculate line number for match
+                curline += content.count('\n', curpos, match.start())
+                curpos = match.start()
+                # add a warning message
+                msg = _("Found %(match)r at line %(line)d in link contents.")
+                self.add_warning(msg %
+                   {"match": match.group(), "line": curline},
+                   tag=WARN_URL_WARNREGEX_FOUND)
+                # check for maximum number of warnings
+                if num >= config["warningregex_max"]:
+                    break
         except tuple(ExcList):
             value = self.handle_exception()
             self.set_result(unicode_safe(value), valid=False)
@@ -762,9 +787,7 @@ class UrlBase (object):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the local HTML tidy module."""
         if not fileutil.has_module("tidy"):
-            log.warn(LOG_CHECK, _("warning: tidy module is not available; " \
-                 "download from http://utidylib.berlios.de/"))
-            return
+            return self.check_html_w3()
         import tidy
         options = dict(output_html=0, show_warnings=1, quiet=True,
             input_encoding='utf8', output_encoding='utf8', tidy_mark=0)
@@ -781,19 +804,15 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: tidy HTML parsing caused error: %(msg)s ") %
+                _("tidy HTML parsing caused error: %(msg)s ") %
                 {"msg": err})
 
     def check_css (self):
         """Check CSS syntax of this page (which is supposed to be CSS)
         with the local cssutils module."""
-        try:
-            import cssutils
-        except ImportError:
-            log.warn(LOG_CHECK,
-                _("warning: cssutils module is not available; " \
-                 "download from http://cthedot.de/cssutils/"))
-            return
+        if not fileutil.has_module("cssutils"):
+            return self.check_css_w3()
+        import cssutils
         try:
             csslog = logging.getLogger('cssutils')
             csslog.propagate = 0
@@ -813,9 +832,24 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: cssutils parsing caused error: %(msg)s") %
+                _("cssutils parsing caused error: %(msg)s") %
                 {"msg": err})
 
+    def check_w3_errors (self, xml, w3type):
+        """Add warnings for W3C HTML or CSS errors in xml format.
+        w3type is either "W3C HTML" or "W3C CSS"."""
+        from xml.dom.minidom import parseString
+        dom = parseString(xml)
+        for error in dom.getElementsByTagName('m:error'):
+            warnmsg = _("%(w3type)s validation error at line %(line)s col %(column)s: %(msg)s")
+            attrs = {
+                "w3type": w3type,
+                "line": getXmlText(error, "m:line"),
+                "column": getXmlText(error, "m:col"),
+                "msg": getXmlText(error, "m:message"),
+            }
+            self.add_warning(warnmsg % attrs)
+
     def check_html_w3 (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the online W3C HTML validator documented at
@@ -826,22 +860,19 @@ class UrlBase (object):
             u = urllib2.urlopen('http://validator.w3.org/check',
                 urllib.urlencode({
                     'fragment': self.get_content(),
-                    'output': 'xml',
+                    'output': 'soap12',
                 }))
             if u.headers.get('x-w3c-validator-status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid HTML syntax"))
                 return
-            from xml.dom.minidom import parseString
-            dom = parseString(u.read())
-            elements = dom.getElementsByTagName('messages')[0].getElementsByTagName('msg')
-            for msg in [e.firstChild.wholeText for e in elements]:
-                self.add_warning(u"W3C HTML validation: %s" % msg)
+            self.check_w3_errors(u.read(), "W3C HTML")
         except Exception:
+            raise
             # catch _all_ exceptions since we dont want third party module
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: HTML W3C validation caused error: %(msg)s ") %
+                _("HTML W3C validation caused error: %(msg)s ") %
                 {"msg": err})
 
     def check_css_w3 (self):
@@ -870,17 +901,13 @@ class UrlBase (object):
             if r.getheader('X-W3C-Validator-Status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid CSS syntax"))
                 return
-            from xml.dom.minidom import parseString
-            dom = parseString(r.read())
-            elements = dom.getElementsByTagName('m:errors')[0].getElementsByTagName('m:error')
-            for msg in [e.firstChild.wholeText for e in elements]:
-                self.add_warning(u"W3C HTML validation: %s" % msg)
+            self.check_w3_errors(r.read(), "W3C HTML")
         except Exception:
             # catch _all_ exceptions since we dont want third party module
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: CSS W3C validation caused error: %(msg)s ") %
+                _("CSS W3C validation caused error: %(msg)s ") %
                 {"msg": err})
 
     def scan_virus (self):
@@ -902,6 +929,9 @@ class UrlBase (object):
         """Get tuple (user, password) from configured authentication.
         Both user and password can be None.
         """
+        if self.userinfo:
+            # URL itself has authentication info
+            return urllib.splitpasswd(self.userinfo)
         return self.aggregate.config.get_user_password(self.url)
 
     def parse_html (self):
@@ -912,6 +942,8 @@ class UrlBase (object):
         # construct parser object
         handler = linkparse.LinkFinder(self.add_url)
         parser = htmlsax.parser(handler)
+        if self.charset:
+            parser.encoding = self.charset
         handler.parser = parser
         # parse
         try:
@@ -923,12 +955,15 @@ class UrlBase (object):
         handler.parser = None
         parser.handler = None
 
-    def add_url (self, url, line, column, name, base):
+    def add_url (self, url, line=0, column=0, name=u"", base=None):
         """Queue URL data for checking."""
-        base_ref = urlutil.url_norm(base)[0]
+        if base:
+            base_ref = urlutil.url_norm(base)[0]
+        else:
+            base_ref = None
         url_data = get_url_from(url, self.recursion_level+1, self.aggregate,
             parent_url=self.url, base_ref=base_ref, line=line, column=column,
-            name=name)
+            name=name, parent_content_type=self.content_type)
         self.aggregate.urlqueue.put(url_data)
 
     def parse_opera (self):
@@ -936,24 +971,25 @@ class UrlBase (object):
         log.debug(LOG_CHECK, "Parsing Opera bookmarks %s", self)
         from ..bookmarks.opera import parse_bookmark_data
         for url, name, lineno in parse_bookmark_data(self.get_content()):
-            url_data = get_url_from(url, self.recursion_level+1,
-               self.aggregate, parent_url=self.url, line=lineno, name=name)
-            self.aggregate.urlqueue.put(url_data)
+            self.add_url(url, line=lineno, name=name)
 
     def parse_chromium (self):
         """Parse a Chromium or Google Chrome bookmark file."""
         log.debug(LOG_CHECK, "Parsing Chromium bookmarks %s", self)
         from ..bookmarks.chromium import parse_bookmark_data
         for url, name in parse_bookmark_data(self.get_content()):
-            url_data = get_url_from(url, self.recursion_level+1,
-               self.aggregate, parent_url=self.url, name=name)
-            self.aggregate.urlqueue.put(url_data)
+            self.add_url(url, name=name)
+
+    def parse_safari (self):
+        """Parse a Safari bookmark file."""
+        log.debug(LOG_CHECK, "Parsing Safari bookmarks %s", self)
+        from ..bookmarks.safari import parse_bookmark_data
+        for url, name in parse_bookmark_data(self.get_content()):
+            self.add_url(url, name=name)
 
     def parse_text (self):
-        """
-        Parse a text file with on url per line; comment and blank
-        lines are ignored.
-        """
+        """Parse a text file with one url per line; comment and blank
+        lines are ignored."""
         log.debug(LOG_CHECK, "Parsing text %s", self)
         lineno = 0
         for line in self.get_content().splitlines():
@@ -961,10 +997,7 @@ class UrlBase (object):
             line = line.strip()
             if not line or line.startswith('#'):
                 continue
-            url_data = get_url_from(line,
-                              self.recursion_level+1, self.aggregate,
-                              parent_url=self.url, line=lineno)
-            self.aggregate.urlqueue.put(url_data)
+            self.add_url(line, line=lineno)
 
     def parse_css (self):
         """
@@ -979,20 +1012,14 @@ class UrlBase (object):
             for mo in linkfinder(line):
                 column = mo.start("url")
                 url = strformat.unquote(mo.group("url").strip())
-                url_data = get_url_from(url,
-                             self.recursion_level+1, self.aggregate,
-                             parent_url=self.url, line=lineno, column=column)
-                self.aggregate.urlqueue.put(url_data)
+                self.add_url(url, line=lineno, column=column)
 
     def parse_swf (self):
         """Parse a SWF file for URLs."""
         linkfinder = linkparse.swf_url_re.finditer
         for mo in linkfinder(self.get_content()):
             url = mo.group()
-            url_data = get_url_from(url,
-                         self.recursion_level+1, self.aggregate,
-                         parent_url=self.url)
-            self.aggregate.urlqueue.put(url_data)
+            self.add_url(url)
 
     def parse_word (self):
         """Parse a word file for hyperlinks."""
@@ -1006,10 +1033,7 @@ class UrlBase (object):
                 doc = winutil.open_wordfile(app, filename)
                 try:
                     for link in doc.Hyperlinks:
-                        url_data = get_url_from(link.Address,
-                                 self.recursion_level+1, self.aggregate,
-                                 parent_url=self.url, name=link.TextToDisplay)
-                        self.aggregate.urlqueue.put(url_data)
+                        self.add_url(link.Address, name=link.TextToDisplay)
                 finally:
                     winutil.close_wordfile(doc)
             finally:
@@ -1020,11 +1044,13 @@ class UrlBase (object):
     def get_temp_filename (self):
         """Get temporary filename for content to parse."""
         # store content in temporary file
-        fd, filename = fileutil.get_temp_file(suffix='.doc', prefix='lc_')
+        fd, filename = fileutil.get_temp_file(mode='wb', suffix='.doc',
+            prefix='lc_')
         try:
             fd.write(self.get_content())
         finally:
             fd.close()
+        return filename
 
     def serialized (self):
         """
