@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,18 +27,18 @@ import urlparse
 import shutil
 import _LinkChecker_configdata as configdata
 from .. import (log, LOG_CHECK, LOG_ROOT, ansicolor, lognames, clamav,
-    get_config_dir, fileutil)
+    get_config_dir, fileutil, configdict)
 from . import confparse
 from ..decorators import memoized
 
 Version = configdata.version
 ReleaseDate = configdata.release_date
-AppName = configdata.appname
+AppName = configdata.name
 App = AppName+u" "+Version
 Author = configdata.author
 HtmlAuthor = Author.replace(u' ', u'&nbsp;')
-Copyright = u"Copyright (C) 2000-2011 "+Author
-HtmlCopyright = u"Copyright &copy; 2000-2011 "+HtmlAuthor
+Copyright = u"Copyright (C) 2000-2012 "+Author
+HtmlCopyright = u"Copyright &copy; 2000-2012 "+HtmlAuthor
 AppInfo = App+u"              "+Copyright
 HtmlAppInfo = App+u", "+HtmlCopyright
 Url = configdata.url
@@ -50,13 +50,13 @@ Freeware = AppName+u""" comes with ABSOLUTELY NO WARRANTY!
 This is free software, and you are welcome to redistribute it
 under certain conditions. Look at the file `LICENSE' within this
 distribution."""
-
+Portable = configdata.portable
 
 def normpath (path):
-    """
-    Norm given system path with all available norm funcs in os.path.
-    """
-    return os.path.normcase(os.path.normpath(os.path.expanduser(path)))
+    """Norm given system path with all available norm or expand functions
+    in os.path."""
+    expanded = os.path.expanduser(os.path.expandvars(path))
+    return os.path.normcase(os.path.normpath(expanded))
 
 
 # List optional Python modules in the form (module, name)
@@ -65,7 +65,8 @@ Modules = (
     ("optcomplete", u"Optcomplete"),
     ("tidy", u"HTMLtidy"),
     ("cssutils", u"CSSutils"),
-    ("GeoIP", u"GeoIP"),
+    ("GeoIP", u"GeoIP"),   # on Unix systems
+    ("pygeoip", u"GeoIP"), # on Windows systems
     ("twill", u"Twill"),
     ("sqlite3", u"Sqlite"),
     ("gconf", u"Gconf"),
@@ -120,6 +121,7 @@ class Configuration (dict):
         self["wait"] = 0
         self['sendcookies'] = False
         self['storecookies'] = False
+        self['cookiefile'] = None
         self["status"] = False
         self["status_wait_seconds"] = 5
         self["fileoutput"] = []
@@ -179,6 +181,7 @@ class Configuration (dict):
         self['output'] = 'text'
         self['logger'] = None
         self["warningregex"] = None
+        self["warningregex_max"] = 5
         self["warnsizebytes"] = None
         self["nntpserver"] = os.environ.get("NNTP_SERVER", None)
         self["threads"] = 10
@@ -186,24 +189,23 @@ class Configuration (dict):
         self["timeout"] = 60
         self["checkhtml"] = False
         self["checkcss"] = False
-        self["checkhtmlw3"] = False
-        self["checkcssw3"] = False
         self["scanvirus"] = False
         self["clamavconf"] = clamav.canonical_clamav_conf()
         self["useragent"] = UserAgent
+        self["debugmemory"] = False
+        self["localwebroot"] = None
+        from ..logger import Loggers
+        self.loggers = dict(**Loggers)
 
     def init_logging (self, status_logger, debug=None, handler=None):
         """
-        Load logging.conf file settings to set up the
-        application logging (not to be confused with check loggers).
-        When debug is not None it is expected to be a list of
+        Set up the application logging (not to be confused with check
+        loggers). When debug is not None it is expected to be a list of
         logger names for which debugging will be enabled.
 
         If no thread debugging is enabled, threading will be disabled.
         """
-        filename = normpath(os.path.join(get_config_dir(), "logging.conf"))
-        if os.path.isfile(filename):
-            logging.config.fileConfig(filename)
+        logging.config.dictConfig(configdict)
         if handler is None:
             handler = ansicolor.ColoredStreamHandler(strm=sys.stderr)
         self.add_loghandler(handler)
@@ -251,8 +253,7 @@ class Configuration (dict):
         args = {}
         args.update(self[loggertype])
         args.update(kwargs)
-        from ..logger import Loggers
-        return Loggers[loggertype](**args)
+        return self.loggers[loggertype](**args)
 
     def logger_add (self, loggertype, loggerclass, loggerargs=None):
         """
@@ -260,8 +261,7 @@ class Configuration (dict):
         """
         if loggerargs is None:
             loggerargs = {}
-        from ..logger import Loggers
-        Loggers[loggertype] = loggerclass
+        self.loggers[loggertype] = loggerclass
         self[loggertype] = loggerargs
 
     def read (self, files=None):
@@ -286,7 +286,7 @@ class Configuration (dict):
         """Add given authentication data."""
         if not user or not pattern:
             log.warn(LOG_CHECK,
-            _("warning: missing user or URL pattern in authentication data."))
+            _("missing user or URL pattern in authentication data."))
             return
         entry = dict(
             user=user,
@@ -312,13 +312,9 @@ class Configuration (dict):
             self.sanitize_anchors()
         if self['logger'] is None:
             self.sanitize_logger()
-        if self['checkhtml']:
-            self.sanitize_checkhtml()
-        if self['checkcss']:
-            self.sanitize_checkcss()
         if self['scanvirus']:
             self.sanitize_scanvirus()
-        if self['storecookies']:
+        if self['storecookies'] or self['cookiefile']:
             self.sanitize_cookies()
         if self['loginurl']:
             self.sanitize_loginurl()
@@ -336,41 +332,27 @@ class Configuration (dict):
     def sanitize_logger (self):
         """Make logger configuration consistent."""
         if not self['output']:
-            log.warn(LOG_CHECK, _("warning: activating text logger output."))
+            log.warn(LOG_CHECK, _("activating text logger output."))
             self['output'] = 'text'
         self['logger'] = self.logger_new(self['output'])
 
-    def sanitize_checkhtml (self):
-        """Ensure HTML tidy is installed for checking HTML."""
-        if not fileutil.has_module("tidy"):
-            log.warn(LOG_CHECK,
-                _("warning: tidy module is not available; " \
-                 "download from http://utidylib.berlios.de/"))
-            self['checkhtml'] = False
-
-    def sanitize_checkcss (self):
-        """Ensure cssutils is installed for checking CSS."""
-        if not fileutil.has_module("cssutils"):
-            log.warn(LOG_CHECK,
-                _("warning: cssutils module is not available; " \
-                 "download from http://cthedot.de/cssutils/"))
-            self['checkcss'] = False
-
     def sanitize_scanvirus (self):
         """Ensure clamav is installed for virus checking."""
         try:
             clamav.init_clamav_conf(self['clamavconf'])
         except clamav.ClamavError:
             log.warn(LOG_CHECK,
-                _("warning: Clamav could not be initialized"))
+                _("Clamav could not be initialized"))
             self['scanvirus'] = False
 
     def sanitize_cookies (self):
         """Make cookie configuration consistent."""
         if not self['sendcookies']:
-            log.warn(LOG_CHECK, _("warning: activating sendcookies " \
-                                  "because storecookies is active."))
+            log.warn(LOG_CHECK, _("activating sendcookies."))
             self['sendcookies'] = True
+        if not self['storecookies']:
+            log.warn(LOG_CHECK, _("activating storecookies."))
+            self['storecookies'] = True
 
     def sanitize_loginurl (self):
         """Make login configuration consistent."""
@@ -378,26 +360,26 @@ class Configuration (dict):
         disable = False
         if not self["loginpasswordfield"]:
             log.warn(LOG_CHECK,
-            _("warning: no CGI password fieldname given for login URL."))
+            _("no CGI password fieldname given for login URL."))
             disable = True
         if not self["loginuserfield"]:
             log.warn(LOG_CHECK,
-            _("warning: no CGI user fieldname given for login URL."))
+            _("no CGI user fieldname given for login URL."))
             disable = True
         if self.get_user_password(url) == (None, None):
             log.warn(LOG_CHECK,
-            _("warning: no user/password authentication data found for login URL."))
+            _("no user/password authentication data found for login URL."))
             disable = True
         if not url.lower().startswith(("http:", "https:")):
-            log.warn(LOG_CHECK, _("warning: login URL is not a HTTP URL."))
+            log.warn(LOG_CHECK, _("login URL is not a HTTP URL."))
             disable = True
         urlparts = urlparse.urlsplit(url)
         if not urlparts[0] or not urlparts[1] or not urlparts[2]:
-            log.warn(LOG_CHECK, _("warning: login URL is incomplete."))
+            log.warn(LOG_CHECK, _("login URL is incomplete."))
             disable = True
         if disable:
             log.warn(LOG_CHECK,
-              _("warning: disabling login URL %(url)s.") % {"url": url})
+              _("disabling login URL %(url)s.") % {"url": url})
             self["loginurl"] = None
         elif not self['storecookies']:
             # login URL implies storing and sending cookies
@@ -419,24 +401,27 @@ class Configuration (dict):
 
 
 def get_user_config():
-    """Find user configuration file.
-    Generate user configuration file from the example configuration
-    if it does not yet exist.
-    Returns path to user config file"""
-    # example config
-    exampleconf = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
+    """Get the user configuration filename.
+    If the user configuration file does not exist, copy it from the initial
+    configuration file, but only if this is not a portable installation.
+    Returns path to user config file (which might not exist do to copy
+    failures or on portable systems)."""
+    # initial config (with all options explained)
+    initialconf = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
     # per user config settings
     userconf = normpath("~/.linkchecker/linkcheckerrc")
-    if os.path.isfile(exampleconf) and not os.path.exists(userconf):
-        # copy the system configuration to the user configuration
+    if os.path.isfile(initialconf) and not os.path.exists(userconf) and \
+       not Portable:
+        # copy the initial configuration to the user configuration
         try:
             userdir = os.path.dirname(userconf)
-            if not os.path.exists(userdir):
-                os.makedirs(userdir)
-            shutil.copy(exampleconf, userconf)
-        except StandardError:
-            log.warn(LOG_CHECK, "could not copy example config from %r to %r",
-                     exampleconf, userconf, traceback=True)
+            if not os.path.isdir(userdir):
+                os.mkdir(userdir, 0700)
+            shutil.copy(initialconf, userconf)
+        except StandardError, errmsg:
+            msg = _("could not copy initial configuration file %(src)r to %(dst)r: %(errmsg)r")
+            args = dict(src=initialconf, dst=userconf, errmsg=errmsg)
+            log.warn(LOG_CHECK, msg % args)
     return userconf
 
 
