@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2009 Bastian Kleineidam
+# Copyright (C) 2006-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,9 +31,28 @@ class CheckedTask (threader.StoppableThread):
             log.warn(LOG_CHECK, "interrupt did not reach the main thread")
             thread.interrupt_main()
         except Exception:
-            console.internal_error()
+            self.internal_error()
 
     @notimplemented
     def run_checked (self):
         """Overload in subclass."""
         pass
+
+    @notimplemented
+    def internal_error (self):
+        """Overload in subclass."""
+        pass
+
+
+class LoggedCheckedTask (CheckedTask):
+    """URL check task with a logger instance and internal error handling."""
+
+    def __init__ (self, logger):
+        """Initialize super instance and store given logger."""
+        super(CheckedTask, self).__init__()
+        self.logger = logger
+
+    def internal_error (self):
+        """Log an internal error on console and the logger."""
+        console.internal_error()
+        self.logger.log_internal_error()
