@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,19 +11,19 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 A blacklist logger.
 """
 
 import os
-
-import linkcheck.logger
+import codecs
+from . import Logger
 
 
-class BlacklistLogger (linkcheck.logger.Logger):
+class BlacklistLogger (Logger):
     """
     Updates a blacklist of wrong links. If a link on the blacklist
     is working (again), it is removed from the list. So after n days
@@ -71,23 +71,21 @@ class BlacklistLogger (linkcheck.logger.Logger):
         """
         Read a previously stored blacklist from file fd.
         """
-        fd = open(self.filename)
-        try:
+        with codecs.open(self.filename, 'r', self.output_encoding,
+                         self.codec_errors) as fd:
             for line in fd:
-                line = self.decode(line.rstrip())
+                line = line.rstrip()
                 if line.startswith('#') or not line:
                     continue
                 value, key = line.split(None, 1)
                 self.blacklist[key] = int(value)
-        finally:
-            fd.close()
 
     def write_blacklist (self):
         """
         Write the blacklist.
         """
         oldmask = os.umask(0077)
-        for key, value in self.blacklist.iteritems():
+        for key, value in self.blacklist.items():
             self.write(u"%d %s%s" % (value, key, os.linesep))
         self.close_fileoutput()
         # restore umask
